#!/opt/uberspace/python-venv/bin/python
import socket

import uberspace_takeout.items as items
import uberspace_takeout.storage as storage
from uberspace_takeout.exc import TakeoutError


__version__ = "0.2.1"


class Takeout:
    takeout_menu = [
        items.common.TakeoutMarker,
        items.common.Homedir,
        items.common.Www,
        items.common.Cronjobs,
        items.common.MySQLPassword,
        items.u7.WebDomains,
        items.u7.MailDomains,
        items.u7.AccessLogItem,
        items.u7.ApacheErrorLogItem,
        items.u7.PhpErrorLogItem,
        items.u7.SpamfilterLogItem,
        items.u7.ToolVersions,
        items.u6.WebDomains,
        items.u6.MailDomains,
    ]

    def __init__(self, hostname=None):
        if not hostname:
            hostname = socket.getfqdn()

        self.hostname = hostname
        self.errors = {}

    def get_items(self, username, storage):
        for item in self.takeout_menu:
            instance = item(username, self.hostname, storage)
            if instance.is_active():
                yield instance

    def takein(self, tar_path, username, skipped_items=None):
        if skipped_items is None:
            skipped_items = []
        with storage.TarStorage(tar_path, "takein") as stor:
            for item in self.get_items(username, stor):
                if item.__class__.__name__ in skipped_items:
                    print("skip: " + item.description)
                    continue

                print("takein: " + item.description)
                try:
                    item.takein()
                except TakeoutError as exc:
                    self.errors[item.__class__.__name__] = exc.args

    def takeout(self, tar_path, username, skipped_items=None):
        if skipped_items is None:
            skipped_items = []
        with storage.TarStorage(tar_path, "takeout") as stor:
            for item in self.get_items(username, stor):
                if item.__class__.__name__ in skipped_items:
                    print("skip: " + item.description)
                    continue

                print("takeout: " + item.description)
                try:
                    item.takeout()
                except TakeoutError as exc:
                    self.errors[item.__class__.__name__] = exc.args



import os
import re

from .base import TakeoutItem
from .base import UberspaceVersionMixin


class U6Mixin(UberspaceVersionMixin):
    uberspace_version = 6


class DomainItem(U6Mixin, TakeoutItem):
    flag = None
    storage_path = None

    def _find_domains(self):
        domains = self.run_command(["uberspace-list-domains", self.flag])
        domains = set(domains) - {
            self.username + "." + self.hostname,
            "*." + self.username + "." + self.hostname,
        }
        return domains

    def takeout(self):
        text = "\n".join(self._find_domains())
        self.storage.store_text(text, self.storage_path)

    def takein(self):
        text = self.storage.unstore_text(self.storage_path)

        for domain in (d for d in text.split("\n") if d):
            self.run_command(["uberspace-add-domain", self.flag, "-d", domain])


class WebDomains(DomainItem):
    description = "Web Domains"
    flag = "-w"
    storage_path = "conf/domains-web"

    def _find_domains(self):
        domains = super()._find_domains()

        try:
            for candidate in os.listdir('/var/www/virtual/' + self.username):
                if not re.search(r'\.[a-z0-9-]{2,}$', candidate):
                    continue

                domains.add(candidate)
        except OSError:
            pass

        return domains


class MailDomains(DomainItem):
    description = "Mail Domains"
    flag = "-m"
    storage_path = "conf/domains-mail"


class ToolVersion(TakeoutItem):
    storage_path = None
    version_regex = None
    software_command = None

    def _clean_version(self, version):
        return version

    def takeout(self):
        output = self.run_command(self.software_command)
        version = re.search(self.version_regex, output).groups()[0]
        version = self._clean_version(version)
        self.storage.store_text(version, self.storage_path)

    def takein(self):
        pass
        # TODO: print warning


class NodeVersion(ToolVersion):
    storage_path = "conf/tool-version/node"
    version_regex = r"^v([0-9]+)"
    software_command = "node --version"


class RubyVersion(ToolVersion):
    storage_path = "conf/tool-version/ruby"
    version_regex = r"^ruby ([0-9]+\.[0-9]+)"
    software_command = "ruby --version"


class PhpVersion(ToolVersion):
    storage_path = "conf/tool-version/php"
    version_regex = r"^PHP ([0-9]+\.[0-9]+)"
    software_command = "php --version"



import sys

try:
    from setuptools import setup
except ImportError:
    print("This project needs setuptools.", file=sys.stderr)
    print("Please install it using your package-manager or pip.", file=sys.stderr)
    sys.exit(1)

from uberspace_takeout import __version__ as version


setup(
    name="uberspace_takeout",
    version=version,
    description="",
    author="uberspace.de",
    author_email="hallo@uberspace.de",
    url="https://github.com/uberspace/takeout",
    packages=["uberspace_takeout", "uberspace_takeout.items",],
    extras_require={"test": ["pyfakefs>=3.6", "pytest-mock",],},
    entry_points={
        "console_scripts": ["uberspace-takeout=uberspace_takeout.__main__:main"],
    },
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "Intended Audience :: Information Technology",
        "Intended Audience :: System Administrators",
        "Topic :: System :: Systems Administration",
        "Topic :: Security",
        "Topic :: Utilities",
        "Natural Language :: English",
        "Operating System :: POSIX :: Linux",
        "Programming Language :: Python :: 2.7",
    ],
    zip_safe=True,
)
